import math
from pharmacokinetics import _handlers

__all__ = [
    "concentrationAtTime",
]

def concentrationAtTime(dose: float, **kwargs) -> float:
    """Calculates the concentration at a specific time (oral administration).

    :param dose: the dosage administered
    :param **kwargs:
        vd: volume of distribution (default 1)
        bioavailability: fraction of drug absorbed into the systemic circulation (default 1)
        ke: elimination rate constant
        t12: elimination half-life (alternative to `ke`)
        ka: absorption rate constant
        t12abs: absorption half-life
        time_elapsed: time since drug administration
        dose_interval: interval between doses (optional)"""
    ke = _handlers._getPkConstantFromKwargs("ke", "t12", kwargs)
    ka = _handlers._getPkConstantFromKwargs("ka", "t12abs", kwargs)
    vd = kwargs.get("vd", 1)
    bioavailability = kwargs.get("bioavailability", 1)
    time_elapsed = kwargs.get("time_elapsed")
    dose_interval = kwargs.get("dose_interval")
    _lFrac = (bioavailability * dose * ka) / (vd * (ka - ke))
    if dose_interval != None:
        _rFrac1 = math.e**(-ke * time_elapsed) / (1 - math.e**(-ke * dose_interval))
        _rFrac2 = math.e**(-ka * time_elapsed) / (1 - math.e**(-ka * dose_interval))
        return _lFrac * (_rFrac1 - _rFrac2)
    return _lFrac * (math.e**(-ke*time_elapsed) - math.e**(-ka * time_elapsed))



import math
from pharmacokinetics import _handlers

__all__ = [
    "calculateTmax",
]

def calculateTmax(**kwargs) -> float:
    """Calculates the time to reach maximum concentration (tmax).

    :param **kwargs:
        dose_interval: the interval between dose administration (optional)
        ka: absorption rate constant
        t12abs: absorption half-life (alternative to `ka`)
        ke: elimination rate constant
        t12: elimination half-life (alternative to `ke`)"""
    ka = _handlers._getPkConstantFromKwargs("ka", "t12abs", kwargs)
    ke = _handlers._getPkConstantFromKwargs("ke", "t12", kwargs)
    dose_interval = kwargs.get("dose_interval")
    isMultipleDoses = dose_interval != None
    if isMultipleDoses:
        _numerator = ka * (1 - math.e**(-ke*dose_interval))
        _denominator = ke * (1 - math.e**(-ka*dose_interval))
        return math.log(_numerator/_denominator) / (ka-ke)
    else:
        return 1 / (ka - ke) * math.log(ka/ke)



import math

__all__ = [
    "halflifeFromDoses",
]

def halflifeFromDoses(initial_dose, remaining_dose, time_elapsed) -> float:
    """Calculates the half-life based on the dose remaining after a specific time.

    :param initial_dose: the initial dose administered
    :param remaining_dose: the remaining dose at the specified time
    :param time_elapsed: the time elapsed since the drug has reached peak concentrations"""
    halfLivesPassed = math.log10(remaining_dose/initial_dose) / math.log10(0.5)
    return time_elapsed / halfLivesPassed



import setuptools
import pharmacokinetics

with open("README.md", "r") as f:
    readme = f.read()

setuptools.setup(
    name="pharmacokinetics",
    version=pharmacokinetics.__version__,
    description=pharmacokinetics.__description__,
    long_description=readme,
    long_description_content_type="text/markdown",
    author=pharmacokinetics.__author__,
    url="https://github.com/xyzpw/pharmacokinetics-module/",
    classifiers=[
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
    ],
    keywords=[
        "pharmacokinetics",
        "pharmacodynamics",
        "pharmacology",
        "pharmacy",
        "chemistry",
    ],
    license="MIT",
)
